/* 
 * tclAppInit.c --
 *
 *  Provides a default version of the main program and Tcl_AppInit
 *  procedure for Tcl applications (without Tk).  Note that this
 *  program must be built in Win32 console mode to work properly.
 *
 * Copyright (c) 1996-1997 by Sun Microsystems, Inc.
 * Copyright (c) 1998-1999 by Scriptics Corporation.
 * Copyright (c) 2000 Jean-Claude Wippler <jcw@equi4.com>
 *
 * See the file "license.terms" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 * RCS: @(#) $Id: kitInit.c,v 1.1 2000/11/08 22:47:57 jcw Exp $
 */

#ifdef KIT_INCLUDES_TK
#include <tk.h>
#else
#include <tcl.h>
#endif

#ifdef _WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#undef WIN32_LEAN_AND_MEAN
#endif

#include "tclInt.h"

extern int  Mk4tcl_Init _ANSI_ARGS_((Tcl_Interp *interp));
extern int  Pink_Init _ANSI_ARGS_((Tcl_Interp *interp));
extern int  Vfs_Init _ANSI_ARGS_((Tcl_Interp *interp));
extern int  Rechan_Init _ANSI_ARGS_((Tcl_Interp *interp));
#ifdef TCL_THREADS
extern int  TclThread_Init _ANSI_ARGS_((Tcl_Interp *interp));
#endif
Tcl_AppInitProc	Pwb_Init;

    /*
     *  Attempt to load a "boot.tcl" entry from the embedded MetaKit file.
     *  If there isn't one, try to open a regular "setup.tcl" file instead.
     *  If that fails, this code will throw an error, using a message box.
     */

static char preInitCmd[] = 
"proc tclKitInit {} {\n"
    "load {} Mk4tcl\n"
    "mk::file open exe [info nameofexecutable] -readonly\n"
    "set n [mk::select exe.dirs!0.files name boot.tcl]\n"
    "if {$n != \"\"} {\n"
        "set s [mk::get exe.dirs!0.files!$n contents]\n"
        "catch {load {} pink; set s [pink zlib decompress $s]}\n"
    "} else {\n"
        "set f [open setup.tcl]\n"
        "set s [read $f]\n"
        "close $f\n"
    "}\n"
    "uplevel #0 $s\n"
    "rename tclKitInit {}\n"
"}\n"
"tclKitInit"
;

static char initScript[] =
"if { [file isfile [file join [info nameofexe] main.tcl]] } {\n"
    "# Embedded script will decide our fate\n"
    "source [file join [info nameofexe] main.tcl]\n"
"} elseif { $tcl_startup != {} && \
	[file isfile [file join [info nameofexe] bin $tcl_startup.tcl]] } {\n"
    "source [file join [info nameofexe] bin $tcl_startup.tcl]\n"
"} elseif { $tcl_startup != {} } {\n"
    "source $tcl_startup\n"
"} else {\n"
    "# Console please\n"
    "continue\n"
"}\n"
;

int TclKit_AppInit(Tcl_Interp *interp)
{
    Tcl_StaticPackage(0, "Mk4tcl", Mk4tcl_Init, NULL);
    Tcl_StaticPackage(0, "pink", Pink_Init, NULL);
    Tcl_StaticPackage(0, "vfs", Vfs_Init, NULL);
    Tcl_StaticPackage(0, "rechan", Rechan_Init, NULL);
#ifdef TCL_THREADS
    Tcl_StaticPackage(0, "Thread", TclThread_Init, NULL);
#endif
    Tcl_StaticPackage(0, "pwb", Pwb_Init, NULL);

    TclSetPreInitScript(preInitCmd);
    Tcl_SetVar(interp, "tcl_main", initScript, TCL_GLOBAL_ONLY);

    if (Tcl_Init(interp) == TCL_ERROR)
        goto error;

#ifdef KIT_INCLUDES_TK
#ifdef _WIN32
    if (Tk_Init(interp) == TCL_ERROR)
        goto error;
    if (Tk_CreateConsoleWindow(interp) == TCL_ERROR)
        goto error;
    Tcl_StaticPackage(interp, "Tk", Tk_Init, Tk_SafeInit);
#else
    Tcl_StaticPackage(0, "Tk", Tk_Init, Tk_SafeInit);
#endif
#endif

    Tcl_SetVar(interp, "tcl_rcFileName", "~/tclkitrc.tcl", TCL_GLOBAL_ONLY);
    return TCL_OK;

error:
#ifdef KIT_INCLUDES_TK
#ifdef _WIN32
    MessageBeep(MB_ICONEXCLAMATION);
    MessageBox(NULL, Tcl_GetStringResult(interp), "Error in TclKit",
        MB_ICONSTOP | MB_OK | MB_TASKMODAL | MB_SETFOREGROUND);
    ExitProcess(1);
    /* we won't reach this, but we need the return */
#endif
#endif
    return TCL_ERROR;
}
