// C++ command extension
//
// Copyright (c) 1999 Jean-Claude Wippler and Equi4 Software.

// DLL nastiness:

#ifndef wDLL
    #ifdef _USRDLL
        #ifdef MINO_EXPORTS
            #define wDLL      __declspec(dllexport)
        #else
            #define wDLL      __declspec(dllimport)
        #endif
    #else
        #define wDLL          
    #endif
#endif

#ifndef wDLLSPEC
    #ifdef _MSC_VER
        #define wDLLSPEC(t)   wDLL t
    #else
        #define wDLLSPEC(t)   t wDLL
    #endif
#endif

#if (defined (_WIN32) || defined (__WIN32__)) && !defined(STATIC_BUILD) && \
        (defined (_MSC_VER) || (defined (__GNUC__) && defined (__declspec)))
    #define wEXPORT(t) extern "C" __declspec(dllexport) t
#else
    #define wEXPORT(t) extern "C" t
#endif

// defined below:

    class wDLL wRef;
        class wDLL wSeqRef;
    class wDLL wCallFrame;
    class wDLL wObjectBase;
        class wDLL wObject;

// not defined here:

    class wContext;
    class wObjImpl;

/////////////////////////////////////////////////////////////////////////////

class wRef
{
protected:
    wObjImpl* _ptr;

public:
    wRef (wObjImpl* ptr_ =0);
    ~wRef ();

    wRef (const wRef& obj_);
    void operator= (const wRef& obj_);

    operator wObjImpl* () const;

    wRef (int value_);
    bool IsInt() const;
    operator int () const;

    wRef (long value_);
    bool IsLong() const;
    operator long () const;

    wRef (double value_);
    bool IsDouble() const;
    operator double () const;

    wRef (const char* value_);
    wRef (const void* value_, int length_);
    bool IsString() const;
    int Length() const;
    int ByteLength() const;
    operator const char* () const;
    operator const void* () const;

    bool IsSeq() const;

    wObject& Cast(const char* type_) const;
};

/////////////////////////////////////////////////////////////////////////////

class wSeqRef : public wRef
{
public:
    wSeqRef ();
    wSeqRef (const wRef& obj_);
    void operator= (const wRef& obj_);

    int Size() const;
    wObjImpl* operator[] (int index_) const;
    void SetAt(int index_, const wRef& obj_);

    wSeqRef GetSlice(int begin_, int end_) const;
    void SetSlice(int begin_, int end_, const wSeqRef& list_);

    void Append(const wRef& obj_);
    void Insert(int index_, const wRef& obj_, int count_ =1);
    void Delete(int index_, int count_ =1);
};

/////////////////////////////////////////////////////////////////////////////

class wCallFrame
{
    wContext* _context;
    wObjImpl** _objv;
    int _objc;
    int _pos;

    wCallFrame (const wCallFrame&); 
    void operator= (const wCallFrame&);

public:
    wCallFrame (wContext* context_, int objc_, wObjImpl** objv_);

    int Lookup(const char** cmds_);
    wRef RegisterCmd(wObject* obj_);

    //int Args() const;
    bool More() const;
    wRef operator++ ();
    wSeqRef Tail();
    friend wDLLSPEC(void) Done(wCallFrame&);

    wCallFrame& operator>> (int& arg_);
    wCallFrame& operator>> (long& arg_);
    wCallFrame& operator>> (double& arg_);
    wCallFrame& operator>> (const char*& arg_);
    wCallFrame& operator>> (wRef& arg_);
    void operator>> (void (*fun_)(wCallFrame&));

    operator wContext* () const;
};

/////////////////////////////////////////////////////////////////////////////

class wObjectBase
{
public:
    void* _token;
    wContext* _context;

protected:
    wObjectBase ();
};

class wObject : public wObjectBase
{
public:
    virtual ~wObject ();

    virtual wRef Call(wCallFrame& cf_);

    bool Register(wContext* context_, const char* name_);
};

/////////////////////////////////////////////////////////////////////////////

wDLLSPEC(int) wDefine(wRef (*)(wCallFrame&), const char*,
                        wContext* =0, const char* ="0.1");

/////////////////////////////////////////////////////////////////////////////
