// Copyright (c) 1999-2000 Jean-Claude Wippler <jcw@equi4.com>
//
// Interface to zlib

#include "wcommon.h"

#include <zlib.h>

    // THIS STUFF SHOULD USE ROPES

wRef wZlibCmd(wCallFrame& cf_)
{
    static const char* cmds[] =
    {
        "adler32", "compress", "crc32", "decompress", "deflate",
	"inflate",
	0
    };

    int wbits = -MAX_WBITS;

    switch (cf_.Lookup(cmds))
    {
    case 0: // adler32 str ?start? -> checksum
        {
            wRef input = ++cf_;
            long start = cf_.More() ? (long) ++cf_ : (long) adler32(0, 0, 0);
            return (long) adler32((unsigned long) start,
			(const Bytef*) (const void*) input, input.ByteLength());
        }

    case 1: // compress str ?level? -> data
	wbits = MAX_WBITS;
    case 4: // deflate str ?level? -> data
        {
            wRef input = ++cf_;
            int level = Z_DEFAULT_COMPRESSION;
            cf_ >> level;

            z_stream stream;
            Bytef* buffer = 0;

            int length = input.Length(); // XXX ByteLength perhaps?
            stream.avail_in = (uInt)input.ByteLength();
            stream.next_in = (Bytef*)(const void*) input;

            stream.avail_out = (uInt) length + length / 1000 + 12;
            stream.next_out = buffer = new Bytef [stream.avail_out];

            stream.zalloc = 0;
            stream.zfree = 0;
            stream.opaque = 0;

            int e = deflateInit2(&stream, level, Z_DEFLATED, wbits,
				    MAX_MEM_LEVEL, Z_DEFAULT_STRATEGY);
            if (e != Z_OK)
	    {
                delete [] buffer;
                throw zError(e);
            }
            e = deflate(&stream, Z_FINISH);
            if (e != Z_STREAM_END)
	    {
                deflateEnd(&stream);
                if (e == Z_OK)
		    e = Z_BUF_ERROR;
            }
	    else
                e = deflateEnd(&stream);

            wRef result (buffer, stream.total_out);
            delete [] buffer;

            if (e != Z_OK)
		throw zError(e);
            return result;
        }
	
    case 2: // crc32 str ?start? -> checksum
        {
            wRef input = ++cf_;
            long start = cf_.More() ? (long) ++cf_ : (long) crc32(0, 0, 0);
            return (long) crc32((unsigned long) start,
			(const Bytef*) (const void*) input, input.ByteLength());
        }
	
    case 3: // decompress data ?bufsize? -> str
	wbits = MAX_WBITS;
    case 5: // inflate data ?bufsize? -> str
        {
            wRef input = ++cf_;
            int bufsize = 16 * 1024;
            cf_ >> bufsize;

            z_stream stream;
            Bytef* buffer = 0;

            for (;;)
            {
                stream.zalloc = 0;
                stream.zfree = 0;

                // +1 because ZLIB can "over-request" input (but ignore it)
                stream.avail_in = (uInt) input.ByteLength() +  1;
                stream.next_in = (Bytef*) (const void*) input;

                stream.avail_out = (uInt) bufsize;
                stream.next_out = buffer = new Bytef [stream.avail_out];

                // Negative value suppresses ZLIB header
		int e = inflateInit2(&stream, wbits);
                if (e == Z_OK)
		{
		    e = inflate(&stream, Z_FINISH);
                    if (e != Z_STREAM_END)
		    {
			inflateEnd(&stream);
			if (e == Z_OK)
			    e = Z_BUF_ERROR;
                    }
		    else
			e = inflateEnd(&stream);
                }

                if (e == Z_OK)
                    break;

                delete [] buffer;
                if (e != Z_BUF_ERROR)
		    throw zError(e);

                bufsize *= 2;
            }

            wRef result (buffer, stream.total_out);
            delete [] buffer;
            return result;
        }
    }

    return wRef ();
}
